@@ -9,6 +9,8 @@ from abc import abstractmethod, ABC
 from enum import Enum
 from inspect import signature, Parameter
 from typing import Any
+from threading import RLock
+import inspect
 
 from .symbols import Symbols
 
@@ -30,7 +32,13 @@ class IServiceInfo(ABC):
 class ServiceInfo(IServiceInfo):
     '''generic `IServiceInfo`.'''
 
-    __slots__ = ('_key', '_lifetime', '_cache_value', '_factory', '_service_provider')
+    __slots__ = (
+        '_key', '_lifetime', '_factory',
+        # for not transient
+        '_lock',
+        # for singleton
+        '_cache_value', '_service_provider'
+    )
 
     def __init__(self, service_provider, key, factory, lifetime):
 
@@ -50,27 +58,48 @@ class ServiceInfo(IServiceInfo):
         self._cache_value = None
         self._service_provider = service_provider
 
-        # service_provider is required when lifetime == singleton
-        assert self._service_provider is not None or self._lifetime != LifeTime.singleton
+        if self._lifetime != LifeTime.transient:
+            self._lock = RLock()
+        else:
+            self._lock = None
+
+        if self._lifetime == LifeTime.singleton:
+            # service_provider is required when lifetime == singleton
+            assert self._service_provider is not None
 
     def get(self, provider):
         if self._lifetime is LifeTime.transient:
             return self._factory(provider)
 
         if self._lifetime is LifeTime.scoped:
-            cache = provider[Symbols.cache]
-            if self not in cache:
-                cache[self] = self._factory(provider)
-            return cache[self]
+            return self._from_scoped(provider)
 
         if self._lifetime is LifeTime.singleton:
-            if self._cache_value is None:
-                provider = self._service_provider
-                self._cache_value = (self._factory(provider), )
-            return self._cache_value[0]
+            return self._from_singleton()
 
         raise NotImplementedError(f'what is {self._lifetime}?')
 
+    def _from_scoped(self, provider):
+        cache: dict = provider[Symbols.cache]
+        try:
+            return cache[self]
+        except KeyError:
+            pass
+        with self._lock:
+            try:
+                return cache[self]
+            except KeyError:
+                value = self._factory(provider)
+                cache[self] = value
+                return value
+
+    def _from_singleton(self):
+        if self._cache_value is None:
+            with self._lock:
+                if self._cache_value is None:
+                    self._cache_value = (self._factory(self._service_provider), )
+        return self._cache_value[0]
+
 
 class ProviderServiceInfo(IServiceInfo):
     '''a `IServiceInfo` use for get current `ServiceProvider`.'''
@@ -81,6 +110,19 @@ class ProviderServiceInfo(IServiceInfo):
         return provider
 
 
+class GetAttrServiceInfo(IServiceInfo):
+    '''getattr from current `ServiceProvider`.'''
+
+    __slots__ = ('_attr_info')
+
+    def __init__(self, *attr_info: tuple):
+        super().__init__()
+        self._attr_info = attr_info
+
+    def get(self, provider):
+        return getattr(provider, *self._attr_info)
+
+
 class ValueServiceInfo(IServiceInfo):
     '''a `IServiceInfo` use for get fixed value.'''
 
@@ -115,3 +157,16 @@ class BindedServiceInfo(IServiceInfo):
 
     def get(self, provider):
         return provider[self._target_key]
+
+
+class CallerFrameServiceInfo(IServiceInfo):
+    'a `IServiceInfo` use for get caller frameinfo'
+
+    __slots__ = ()
+
+    def get(self, _):
+        frs = inspect.getouterframes(inspect.currentframe())
+        for fr in frs[2:]:
+            mo = inspect.getmodule(fr.frame)
+            if mo is None or mo.__name__.partition('.')[0] != 'anyioc':
+                return fr
